# HCOMBINE - Combine merged images, removing cosmic rays in the process
# July 20, 2004
# By Doug Mink, SAO Telescope Data Center

procedure hcombine (images, output)

string	images=""	{prompt="Images to combine"}
file	output=""	{prompt="Output image"}
bool	fix = yes	{prompt="Replace cosmic rays using fixpix (y or n)"}
string	combine = "average"	{prompt="Combine images (average|median|sum)"}
string	reject = "none"	{prompt="Reject method (none|minmax|ccdclip|crreject|sigclip|avsigclip|pclip)"}
int	nsig = 4	{prompt="N sigma threshold for cosmic ray difference"}
int	threshold = 30	{prompt="Minimum cosmic ray difference in counts"}
int	mincounts = 30	{prompt="Minimum counts for median"}
bool	newsig = no	{prompt="Is this a new nsig (yes or no)"}
bool	saveold = yes	{prompt="Save uncombined files in Uncombined/ (y or n)"}
bool	deltemp = no	{prompt="Delete intermediate image files (y or n)"}
bool	verbose = yes	{prompt="Print process progress (y or n)"}

begin

int	ifile, nfiles, lname, lroot
string	iml, tchar, iml1, imagelist, cleanlist, imageroot, imagej
string	outroot, extxt, combim, rejim, expression, imin, coslist, history
file	tempmed, tempdiff, tempbad, tempsig, imagei, listfile, imageout, imfile
file	rejfile, secfile, tempnsig, newfile
real	medmed, medi, factor, basemed
bool	dellist, vb, fixim, basefile, newmed

	vb = verbose
	dellist = yes
	imagelist = images
	imageout = output
	newmed = no
	rejim = reject
	combim = combine
	fixim = fix
	smatch (imageout, ".fits")
	if (smatch.found) {
	    outroot = substr (imageout,1,smatch.start-1)
	    }
	else {
	    outroot = imageout
	    }
#	if (vb) {
#	    print ("HCOMBINE: output file ",imageout," root is ",outroot)
#	    }

# Initialize imcombine
	combine.headers = ""
	combine.bpmasks = ""
	combine.nrejmasks = ""
	combine.expmasks = ""
	combine.sigmas = ""
	combine.ccdtype = ""
	combine.amps = no
	combine.subsets = no
	combine.delete = no
	combine.combine = "median"
	combine.reject = "none"
	combine.project = no
	combine.outtype = "real"
	combine.outlimits = ""
	combine.offsets = "none"
	combine.masktype = "none"
	combine.maskvalue = 0.
	combine.blank = 0.
	combine.scale = "none"
	combine.zero = "none"
	combine.weight = "none"
	combine.statsec = ""
	combine.lthreshold = INDEF
	combine.hthreshold = INDEF
	combine.nlow = 1
	combine.nhigh = 1
	combine.nkeep = 1
	combine.mclip = yes
	combine.lsigma = 3.
	combine.hsigma = 3.
	combine.rdnoise = "0."
	combine.gain = "1."
	combine.snoise = "0."
	combine.sigscale = 0.1
	combine.pclip = -0.5
	combine.grow = 0.

#  Make one-per-line list of files from input string or file of filenames

#  If first character of imagelist is @, extract list file name
        tchar = substr (imagelist,1,1)
        if (tchar == "@") {
            listfile = substr (imagelist,2,100)
            dellist = no
            }

#  Otherwise, used files to translate to a file of filenames
        else {
            listfile = mktemp ("hcomb")
            files (imagelist,>listfile)
            dellist = yes
            }
        if (vb)
            print ("HCOMBINE: Reading filenames from ",listfile)

	istat.fields = "image,npix,mean,median,stddev,min,max"
	istat.lower = mincounts
	istat.upper = INDEF
	istat.nclip = 0
	istat.lsigma = 3.
	istat.usigma = 3.
	istat.binwidth = 0.1
	istat.print = no
	istat.format = no
	istat.cache = no

#  Scale images to near-identical flux for easier cosmic ray masking
	coslist = ""
        list = listfile
	basefile = yes
        while (fscan (list, imagei) != EOF) {

#	Find root of filename (without .fits, with _extname)
	    smatch (imagei, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imageroot = substr (imagei, 1, lroot)
		}
	    else {
		imageroot = imagei
		imagei = imagei // ".fits"
		}

	    imagej = imageroot // "x.fits"
	    if (!access (imagej))
		newmed = yes
	    if (basefile || !access (imagej)) {
		istat (imagei)
		if (vb)
		    print (imagei,": ",istat.npix," pixels > ",mincounts, " mean = ",istat.mean, " median = ",istat.median)
		}

#	Get median of data in image and scale to first image
	    if (basefile) {
		if (!access (imagej))
		    imcopy (imagei, imagej)
		basemed = istat.median
		coslist = imagej
		basefile = no
		}
	    else if (!access (imagej)) {
		factor = istat.median / basemed
		imarith (imagei,"*",factor,imagej)
		coslist = coslist // "," // imagej
		history = "Image multiplied by " // factor
		hedit (imagej,"HISTORY",history,addonly=yes,update=yes,verify=no,show=no)
		}
	    else {
		coslist = coslist // "," // imagej
		if (vb)
		    print (imagej," reused")
		}
	    }

#  Make median from scaled input images
	tempmed = outroot // "_med.fits"
	if (access (tempmed) && newmed)
	    imdelete (tempmed)
	if (!access (tempmed)) {
	    combine.combine = "median"
	    combine.reject = "none"
	    combine.outtype="real"
	    combine (coslist, tempmed)
	    if (vb)
		print ("HCOMBINE: Median image ",tempmed," from ",coslist)
	    }
	else if (vb)
	    print ("HCOMBINE: Median image ",tempmed," re-used")

#  Make standard deviation file from median file
	tempsig = outroot // "_sig.fits"
	if (access (tempsig) && (newmed || newsig)) {
	    imdelete (tempsig)
	    }
	if (!access (tempsig)) {
	    expression = nsig // "*sqrt(a)"
	    imexpr (expression, tempsig, tempmed)
	    if (vb)
		print ("HCOMBINE: ", nsig, " sigma error file is ",tempsig)
	    }
	else
	    print ("HCOMBINE: Re-use ", nsig, " sigma error file ",tempsig)

#  Add nsig * standard deviation file to median file
	tempnsig = outroot // "_nsig.fits"
	if (access (tempnsig) && (newmed || newsig)) {
	    imdelete (tempnsig)
	    }
	if (!access (tempnsig))
	    imarith (tempsig, "+", tempmed, tempnsig)

#  Remove cosmic rays from one image at a time
	cleanlist = ""
        list = listfile
        while (fscan (list, imagei) != EOF) {

#  Find root of filename (without .fits, with _extname)
	    smatch (imagei, ".fits")
	    if (smatch.found) {
		lroot = smatch.start - 1
		imageroot = substr (imagei, 1, lroot)
		}
	    else {
		imageroot = imagei
		imagei = imagei // ".fits"
		}

#	Subtract median + n * sigma from scaled copy of current exposure
	    imagej = imageroot // "x.fits"
            tempbad = imageroot // "_bad.fits"
	    if (access (tempbad)) {
		imdelete (tempbad)
		}
	    imarith (imagej,'-', tempnsig, tempbad)
	    if (vb) {
		print ("HCOMBINE: Scaled difference file ", tempbad)
		}

#	Make a bad pixel map for this image
#	All pixels with value less than or equal to threshold will be 0 (good)
#	All pixels with value greater than threshold will be 1 (bad)
	    imreplace (tempbad, 0, upper=threshold)
	    imreplace (tempbad, 1, lower=threshold)
	    if (vb) {
		print ("HCOMBINE: Bad pixel file ", tempbad)
		}

#	Replace cosmic rays with data around them if fix is set
	    if (fixim) {
		imfile = imageroot // "c.fits"
		if (access (imfile)) {
		    imdelete (imfile)
		    }
		imcopy (imagei, imfile)
		fixpix (imfile, tempbad, verbose=vb)
		if (vb) {
		    print ("HCOMBINE: Cleaned image ", imfile)
		    }
		if (deltemp)
		    imdelete (tempbad)
		}
	    else {
		imfile = imageroot // "t.fits"
		imcopy (imagei, imfile)
		hedit (imfile,"BPM",tempbad,addonly=yes,update=yes,verify=no,show=no)
		if (vb) {
		    print ("HCOMBINE: New BPM value in ", imfile)
		    }
		}
	    if (cleanlist == "") {
		cleanlist = imfile
		}
	    else {
		cleanlist = cleanlist // "," // imfile
		}
	    if (saveold) {
		if (!access ("Uncombined/")) {
		    mkdir ("Uncombined")
		    }
		newfile = "Uncombined/" // imagei
		if (access (newfile)) {
		    imdelete (newfile)
		    }
		move (imagei, "Uncombined/")
		if (vb) {
		    print ("HCOMBINE: Original ", imagei, "=> ", newfile)
		    }
		}
	    }

#	Sum fixed images
	combine.combine = combim
	combine.reject = rejim
	if (vb) {
	    print ("HCOMBINE: Combined file ", imageout)
	    }
	combine.outtype="real"
	combine.rejmasks = ""
	combine (cleanlist, imageout)
	if (dellist) {
	    delete (listfile)
	    }
	if (deltemp) {
	    imdelete (tempmed)
	    delete (cleanlist)
	    }
end

# Oct  1 2003	New program to remove cosmic rays from spectra
# Oct  9 2003	Add threshold parameter and use temp files
# Oct 24 2003	Initialize cleanlist concatenation
# Oct 28 2003	Add options to add files with masks
# Oct 31 2003	Add combination and rejection option parameters

# Jun 16 2004	Change name fromZAP to HCOMBINE
# Jun 16 2004	Compute per pixel n*sigma from median file
# Jun 17 2004	Only combine single image files
# Jun 18 2004	Check for existence of all output files; delete if present
# Jun 30 3004	Add lthreshold=mincounts to image statistics
# Jul 15 2004	Scale images before making initial median image
# Jul 16 2004	Use combine instead of imcombine
# Jul 20 2004	Add saveold option to save files in Uncombined/
