# HSPLIT1 - Split a single multispec hectospec file into 1-D spectra
# By Doug Mink, SAO Telescope Data Center
# September 16, 2005

procedure hsplit1 (msfile)

string	msfile=""	{prompt="Aperture reference multispec file (drop .ms)"}
file	keywords="hectospec$lib/keywords" {prompt="File containing header keywords to be deleted"}
bool	deltemp=yes	{prompt="Delete temporary list file (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print more information (y or n)"}

begin

	file	keyfile, keypath, imin, imroot, templist, imnew
	file	tempfile, tempfile1, tempfile2
	int	lroot, aperture, nfiles, i
	string	arg, tchar, ims, history, dir1d
	string	keyword, apkwr, apval1, type, x1, x2, range
	string	name, ra, dec, catnum, fiber, x, y
	string	apkey, fpkey, objkey, skykey, fibkey
	string	tra, tdec, tepoch, field, apname
	bool	vb, db

	vb = verbose
	db = debug
	imin = msfile
	keyfile = osfn (keywords)

	if (imin == "") {
	    print "hsplit: Make 1D spectra (fiber.object) from Hectospec multispec file"
	    return
	    }

	if (!defpac ("wcstools")) {
	    wcstools (motd=no)
	    }

# Make sure that input is a multispec file
	smatch (imin, ".ms")
	if (smatch.found) {
	    lroot = smatch.start - 1
	    imroot = substr (imin, 1, lroot)
	    }
        else {
	    smatch (imin, ".fits"
	    if (smatch.found) {
		lroot = smatch.start - 1
		imroot = substr (imin, 1, lroot)
		imnew = imroot // ".ms.fits"
		if (access (imnew)) {
		    imin = imnew
		    }
		else {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
	    else {
		imroot = imin
		imin = imroot // ".ms.fits"
		if (!access (imin)) {
		    print ("HSPLIT: ", imin," is not a multispec file")
		    return
		    }
		}
            }

# Do not split an image twice
	imgetp.default = "0"
	imgetp (imin, "HSPLIT")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("HSPLIT: ", imin, " has already been split")
		print ("HSPLIT = ", imgetp.value)
		}
	    return
	    }

	if (vb) {
	    print ("Delete keywords listed in file ",imin)
	    }
	if (vb) {
	    print ("Object name and position from ", imin)
	    }

# Make separate file to pre-edit header for output files
	tempfile = imroot // "t.ms.fits"
	if (access (tempfile)) {
	    delete (tempfile)
	    }
	tempfile1 = imroot // "i"

# Copy .fits or .imh output from DOFIBERS into file to be edited
	copy (imin, tempfile)

# Get number of spectra in multispec file
	imgetp.default = "0"
	imgetp (imin, "i_naxis2")
	nfiles = int (imgetp.value)

# Delete unneeded keywords from header (mostly APNUMnn)
	if (vb) {
	    print ("HSPLIT1: delhead ",tempfile," @",keyfile)
	    }
	if (db) {
	    arg = tempfile // " -v @" // keyfile
	    }
	else {
	    arg = tempfile // " @" // keyfile
	    }
	dhead (arg)

# Change header position keywords
	imgetp.default = ""
	imgetp (imin, "RA")
	tra = imgetp.value
	padstr (tra)
	print ("tra = ",tra)
	tra = padstr.value
	imgetp.default = ""
	imgetp (imin, "DEC")
	padstr (imgetp.value)
	tdec = padstr.value
	imgetp.default = ""
	imgetp (imin, "EQUINOX")
	imgetp.default = imgetp.value
	imgetp (imin, "EPOCH")
	tepoch = imgetp.value
	imgetp.default = ""
	imgetp (imin, "OBJECT")
	padstr (imgetp.value)
	field = padstr.value
	skykey = "field=" // field // " type=0 tra=" // tra
	skykey = skykey // " tdec=" // tdec // " tepoch=" // tepoch
	if (vb) {
	    print ("HSPLIT1: ", skykey)
	    }

# Set place-holding keywords in temporary file header
	fpkey = "epoch=2000.0 equinox=2000.0 wcsdim=1 imagetyp=OBJECT"
	apkey = "fiber=0 aprange=0:0 fiber_x=0 fiber_y=0 catname=name catnum=0 aperture=0"
#	print "sethead ",tempfile," ",fpkey," ",skykey," ",apkey
	arg = tempfile // " " // fpkey // " " // skykey // " " // apkey
	sethead (arg)

# If 1d directory does not exist, create it
	dir1d = "1d." // imroot
	if (!access (dir1d)) {
	    if (vb)
		print ("HSPLIT: Making directory ", dir1d)
	    mkdir (dir1d)
	    }

# Extract spectra one at a time
	for (i = 1; i <= nfiles; i = i + 1) {

# Get APNUM keyword value for this spectrum and extract fiber and x range
	    apkwr = "APNUM" // i
	    imgetp.default = "0"
	    imgetp (imin, apkwr)
	    if (imgetp.value == "0") {
		aperture = i
		apkey = "specnum=" // i // "aperture=" // i
		apkey = apkey // "apnum1=" // i // "_" // i
		}
	    else {
		padstr (imgetp.value)
		apval1 = padstr.value

		keyword = apkwr // ",1"
		imgetp.default = "0"
		imgetp (imin, keyword)
		aperture = int (imgetp.value)
		if (db) {
		    print ("Aperture is ",aperture)
		    }

		keyword = apkwr // ",2"
		imgetp.default = "0"
		imgetp (imin, keyword)
		type = imgetp.value

		keyword = apkwr // ",3"
		imgetp.default = "0"
		imgetp (imin, keyword)
		x1 = imgetp.value

		keyword = apkwr // ",4"
		imgetp.default = x1
		imgetp (imin, keyword)
		x2 = imgetp.value
		range = x1 // ":" // x2

		apkey = "specnum=" // i // " aperture=" // aperture
		apkey = apkey // " aprange=" // range // " apnum1=" // apval1
		}

# Get pointing direction from APID keyword
	    apkwr = "APID" // i
	    imgetp.default = "0"
	    imgetp (imin, apkwr)
	    apval = imgetp.value
	    if (vb) {
		print (apkwr, " = ", apval)
		}

	    keyword = apkwr // ",1"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    name = imgetp.value

	    keyword = apkwr // ",2"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    padstr (imgetp.value)
	    ra = padstr.value

	    keyword = apkwr // ",3"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    padstr (imgetp.value)
	    dec = padstr.value

	    keyword = apkwr // ",4"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    catnum = imgetp.value

	    keyword = apkwr // ",5"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    fiber = int (imgetp.value)

	    keyword = apkwr // ",6"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    x = real (imgetp.value)

	    keyword = apkwr // ",7"
	    imgetp.default = "0"
	    imgetp (imin, keyword)
	    y = real (imgetp.value)

	    if (name == "target") {
		objkey = "object=" // field // "_" // catnum // " catobj=target"
		}
	    else {
		cleanstr (name)
		name = cleanstr.value
		objkey = "object=" // name // " catobj=" // name
		}
	    skykey = "type=" // type // " ra=" // ra // " dec=" // dec
	    skykey = skykey // " catnum=" // catnum
	    fibkey = "fiber=" // fiber // " fiber_x=" // x
	    fibkey = fibkey // " fiber_y=" // y // " dispaxis=1"
	    if (db) {
		print (objkey, " ", apkey)
		print (skykey)
		print (fibkey)
		}

# Extract this spectrum into a separate FITS file
	    arg = "-s _ -o " // tempfile1 // " " // tempfile // " " // i
	    arg = arg // " " // objkey
	    arg = arg // " " // skykey // " " // apkey
	    if (vb) {
		print ("imextract ", arg)
		}
	    imextract (arg)

	    tempfile2 = tempfile1 // ".fits"
	    arg = tempfile2 // " " // fibkey
	    sethead (arg)
	    if (vb) {
		print ("sethead ", arg)
		}

# Add history line including time
	    gettime ()
	    history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	    hedit (tempfile1, "HSPLIT", history, add+, verify-, show-)

# Make up new file name from fiber number and object name
	    if (aperture < 10) {
		apname = "00" // aperture
		}
	    else if (aperture < 100) {
		apname = "0" // aperture
		}
	    else {
		apname = aperture
		}
	    if (name == "target") {
		imnew = dir1d // "/" // apname // "." // field // "_"
		imnew = imnew // catnum // ".fits"
		}
	    else {
		imnew = dir1d // "/" // apname // "." // name // "_f"
		imnew = imnew // fiber // ".fits"
		}
	    rename (tempfile2, imnew)
	    if (vb) {
		print ("rename ",tempfile2, " ", imnew)
		}
	    }

	if (deltemp) {
	    delete (tempfile)
	    }

# Add history line including time to original multispec file
	gettime ()
	history = gettime.date // " " // gettime.time // " image split to " // "1d." // imroot
	if (vb) {
	    hedit (imin, "HSPLIT", history, add+, verify-, show+)
	    }
	else {
	    hedit (imin, "HSPLIT", history, add+, verify-, show-)
	    }
	return
end

# Feb 22 2000	New program based on split2df
# May 25 2000	Add code to deal with possible input FITS file

# Oct  9 2003	Add -k option to specify keyword deletion list file
# Oct 28 2003	Update parsing code and rewrite to work on actual data
# Oct 31 2003	Save name from APIDn keyword value; use for filename if not target

# Apr 15 2004	Write APNUM1 keyword to 1D files with APNUMi value

# Mar  7 2005	Turn special characters in objet names into _
# Mar  7 2005	Set dispaxis=1
# Apr 11 2005	Run using /data/oir/bin/ksh instead of /data/mmti/bin/ksh
# Sep  8 2005	Convert to CL script from KSH script
# Sep 16 2005	Send fiberkey to correct output file
