# MAKETEST - Make single Hectospec image from raw 4-amp multiextension image
# June 23, 2004
# By Doug Mink, SAO Telescope Data Center

procedure maketest (image)

string	image=""	{prompt="Image to merge"}

file	imdir = ""	{prompt="Directory wherein image resides"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, temp2, temp3, temp4, imin, imout
file	imfile, imraw
string	amp1, amp2, amp3, amp4, arg1, imx, history
bool	dellist, vb

	vb = verbose
	imx = ".fits"
	imraw = image

	if (imraw == "") {
	    print ("MAKETEST: Combine 4 amplifiers on chips from one Hectospec or Hectochelle image")
	    print ("Usage: mergechips <input image with 4 extensions> <output image>")
	    return
	    }

# Find root name of image
	imroot = imraw
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imraw
	    }
	if (vb) {
	    print ("MAKETEST: image root is ",imroot)
	    }
	imin = imroot // imx
	imraw = imdir // "/" // imin

# Make local copy of raw image
	copy (imraw, imin)

# Trim image and remove bias
	hproc (imin, saveold=no, deltemp=yes, verbose=yes)

#  Split out one image per amplifier
	temp1 = mktemp ("hamp1") // imx
	amp1 = imin // "[1]"
	imcopy (amp1, temp1)
	temp2 = mktemp ("hamp2") // imx
	amp2 = imin // "[2]"
	imcopy (amp2, temp2)
	temp3 = mktemp ("hamp3") // imx
	amp3 = imin // "[3]"
	imcopy (amp3, temp3)
	temp4 = mktemp ("hamp4") // imx
	amp4 = imin // "[4]"
	imcopy (amp4, temp4)

#  Join four amps into a single image
	arg1 = temp1 // "," // temp2 // "[-*,*]," // temp3 // "," // temp4 // "[-*,*]"
	delete (imin)
	imjoin (arg1, imin, 1)

# Make history line including time for preamp correction
        gettime ()
	history = gettime.date // " " // gettime.time // " 4 amplifiers merged into one image"
	hedit (imout, "MAKETEST", history, add+, verify-)

	if (deltemp) {
	    imdelete (temp1)
	    imdelete (temp2)
	    imdelete (temp3)
	    imdelete (temp4)
	    }
end

# Jun 23 2004	New program to make a single image from four amplifiers
